/* daemonitor/src/daemonitor/700_signal.c
 *
 *  Copyright: ©2007–2012, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



/* SIGNAL HANDLING ********************************************************************************
 *
 *  This file contains signal handlers and associated setup routines. Most signals are ignored.
 *  SIGTERM is translated into a SIGTERM of the child process, which we will kill within 3 seconds
 *  if it has not exited.
 */



/* PUBLIC API ************************************************************************************/



/* signal_setup()
 *  Installs the signal handlers.
 */
void signal_setup(void);



/* IMPLEMENTATION ********************************************************************************/



/* sigterm_handler()
 *  Handles receiving SIGTERM. If there is a child process, we set the SIGTERM flag, then return.
 *  Otherwise, we simply exit.
 */
void sigterm_handler(int signum __attribute__((unused)))
{
    if(program_pid) {
        sigterm_received = 1;
        return;
    }

    /* uck -- we can't call exit() because it's not async-signal-safe, so we must call _exit() and
     * clean up manually, with no logging */
    unlink(pidfile_filename);
    _exit(0);
}



/* signal_setup()
 *  Block most signals. Install signal handlers for the remainder.
 */
void signal_setup(void)
{
    sigset_t ss;
    struct sigaction sa;

    sigfillset(&ss);
    memset(&sa, 0, sizeof(sa));

    /* SIGKILL of course can't be blocked; neither can stop/continue */
    sigdelset(&ss, SIGKILL);
    sigdelset(&ss, SIGSTOP);
    sigdelset(&ss, SIGCONT);

    /* program error conditions -- don't block them or we won't crash when we should */
    sigdelset(&ss, SIGILL);
    sigdelset(&ss, SIGFPE);
    sigdelset(&ss, SIGSEGV);
    sigdelset(&ss, SIGBUS);

    /* we'd like to deal with SIGTERM/SIGINT specially */
    sigdelset(&ss, SIGTERM);
    sigdelset(&ss, SIGINT);

    /* don't block SIGCHLD or wait(2) won't work */
    sigdelset(&ss, SIGCHLD);

    sigprocmask(SIG_SETMASK, &ss, 0);

    /* restore default signal dispositions in case we inherited SIG_IGN from parent */
    sa.sa_handler = SIG_DFL;
    sa.sa_flags = SA_NOCLDSTOP;
    sigaction(SIGCONT, &sa, 0);
    sigaction(SIGILL, &sa, 0);
    sigaction(SIGFPE, &sa, 0);
    sigaction(SIGSEGV, &sa, 0);
    sigaction(SIGBUS, &sa, 0);
    sigaction(SIGCHLD, &sa, 0);

    /* install our own handler for SIGTERM/SIGINT */
    sa.sa_handler = sigterm_handler;
    sigaction(SIGTERM, &sa, 0);
    sigaction(SIGINT, &sa, 0);
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
