/* daemonitor/src/daemonitor/800_checks.c
 *
 *  Copyright: ©2012, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



/* RESPAWN CHECKS **************************************************************
 *
 *  This file implements functionality for various checks that may be carried
 *  out whenever the child process needs to be respawned. An arbitrary number of
 *  checks may be registered; if any single check fails, the entire test counts
 *  as failing. The result of the checks is found by calling run_checks().
 *
 *  A failed check will log details at LOG_NOTICE (or above).
 */



/* PUBLIC API *****************************************************************/



/* check_command_register()
 *  cmd: A command (passed to system(3)) to run. If it exits with code 0, the
 *       check passes.
 */
void check_command_register(const char* cmd);



/* check_file_register()
 *  path: A file to test. If it exists, the check passes.
 */
void check_file_register(const char* cmd);



/* run_checks()
 *  Returns 0 if all checks succeeded (including when none are registered) and
 *  non-0 if any check failed.
 */
int run_checks(void);



/* IMPLEMENTATION *************************************************************/



enum check_type {
    check_type_command,
    check_type_file,
};



/* struct check
 * check_head
 *  A linked list of the checks that are to be run.
 */
struct check {
    enum check_type type;

    /* can be the argument for system(3) or the path to check for a file */
    const char* arg;

    /* linked list structure */
    struct check* next;
};

struct check* check_head = 0;



/* check_generic_register()
 *  Instantiates a ‘struct check’ and adds it to the list at ‘check_head’.
 */
void
check_generic_register(enum check_type type, const char* arg)
{
    struct check* c;

    c = malloc(sizeof(*c));
    c->type = type;
    c->arg = arg;
    c->next = check_head;
    check_head = c;
}



void
check_command_register(const char* cmd)
{
    check_generic_register(check_type_command, cmd);
}



int
check_command_run(const struct check* iter)
{
    int ret;

    ret = system(iter->arg);
    if(ret == -1) {
        LOG(LOG_ERR, "Unable to run check command \"%s\": %m.", iter->arg);
        return 0; /* don't exit just because system(3) failed */
    }

    if(WIFSIGNALED(ret)) {
        LOG(LOG_WARNING, "Check command \"%s\" killed by signal (%s). Exiting.",
            iter->arg, strsignal(WTERMSIG(ret)));
        return -1;
    }

    if(!WIFEXITED(ret)) {
        LOG(LOG_WARNING, "Unrecognised exit code 0x%X from check command "
            "\"%s\".", ret, iter->arg);
        return 0;
    }

    if(WEXITSTATUS(ret)) {
        LOG(LOG_NOTICE, "Check command \"%s\" signalled failure (exit code "
            "%d). Exiting.", iter->arg, WEXITSTATUS(ret));
        return -1;
    }

    return 0;
}



void
check_file_register(const char* path)
{
    check_generic_register(check_type_file, path);
}



int
check_file_run(const struct check* iter)
{
    int ret;

    ret = access(iter->arg, F_OK);
    if(ret) {
        LOG(LOG_NOTICE, "Check file \"%s\" does not exist. Exiting.",
            iter->arg);
        return -1;
    }

    return 0;
}



int
run_checks(void)
{
    const struct check* iter;

    for(iter = check_head; iter; iter = iter->next) {
        switch(iter->type) {
        case check_type_command:
            if(check_command_run(iter)) return -1;
            break;

        case check_type_file:
            if(check_file_run(iter)) return -1;
            break;
        }
    }

    return 0;
}



/* options for text editors
vim: expandtab:ts=4:sw=4
*/
